package no.tornado.query;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

public class Mapping {
    private String column;
    private Integer sqlType;
    private Mapper.MapType mapType;
    private JoinTable joinTable;
    private String property;
    private String sequence;
    private Mapper joinMap;

    public Mapping(String property, String column) {
        this.property = property;
        this.column = column;
        this.mapType = Mapper.MapType.TABLE_COLUMN;
    }

    public Mapping(String property, String column, Mapper.MapType mapType) {
        this.property = property;
        this.column = column;
        this.mapType = mapType;
    }

    public String getQualifiedColumn() {
        if (joinTable != null)
            return joinTable.getAlias() + "_" + column;

        return column;
    }

    public String toString() {
        StringBuilder s = new StringBuilder(property + " -> " + getQualifiedColumn() + " " + mapType);
        if (joinTable != null)
            s.append(" (" + joinTable.getAlias() + ")");

        if (sequence != null)
            s.append(", sequence " + sequence);

        return s.toString();
    }
    
    public String getColumn() {
        return column;
    }

    public void setColumn(String column) {
        this.column = column;
    }

    public String getProperty() {
        return property;
    }

    public void setProperty(String property) {
        this.property = property;
    }

    public Mapper.MapType getMapType() {
        return mapType;
    }

    public void setMapType(Mapper.MapType mapType) {
        this.mapType = mapType;
    }

    public JoinTable getJoinTable() {
        return joinTable;
    }

    public void setJoinTable(JoinTable joinTable) {
        this.joinTable = joinTable;
    }

    public Mapping joinTable(JoinTable joinTable) {
        this.joinTable = joinTable;
        return this;
    }

    public String getSequence() {
        return sequence;
    }

    public void setSequence(String sequence) {
        this.sequence = sequence;
    }

    public Mapping sequence(String sequence) {
        setSequence(sequence);
        return this;
    }

    public Integer getSqlType() {
        return sqlType;
    }

    public void setSqlType(Integer sqlType) {
        this.sqlType = sqlType;
    }
    
    public Mapping sqlType(Integer sqlType) {
        setSqlType(sqlType);
        return this;
    }

    public Object getResultObject(ResultSet rs) throws SQLException {
        if (sqlType == null)
            return rs.getObject(getQualifiedColumn());

        switch (sqlType) {
            case Types.INTEGER: return rs.getInt(getQualifiedColumn());
            case Types.VARCHAR: return rs.getString(getQualifiedColumn());
            case Types.BOOLEAN: return rs.getBoolean(getQualifiedColumn());
            case Types.TIMESTAMP: return rs.getTimestamp(getQualifiedColumn());
            case Types.TIME: return rs.getTime(getQualifiedColumn());
            case Types.DATE: return rs.getDate(getQualifiedColumn());
            case Types.DECIMAL: return rs.getFloat(getQualifiedColumn());
            case Types.DOUBLE: return rs.getDouble(getQualifiedColumn());
            case Types.BIGINT: return rs.getLong(getQualifiedColumn());
            case Types.SMALLINT: return rs.getInt(getQualifiedColumn());
            case Types.TINYINT: return rs.getInt(getQualifiedColumn());
            case Types.FLOAT: return rs.getFloat(getQualifiedColumn());
            case Types.NUMERIC: return rs.getFloat(getQualifiedColumn());
            case Types.LONGVARCHAR: return rs.getString(getQualifiedColumn());
            case Types.CHAR: return rs.getString(getQualifiedColumn());
            case Types.BLOB: return rs.getBlob(getQualifiedColumn());
            case Types.ARRAY: return rs.getArray(getQualifiedColumn());
            case Types.CLOB: return rs.getString(getQualifiedColumn());
            case Types.NCLOB: return rs.getString(getQualifiedColumn());
            case Types.LONGNVARCHAR: return rs.getString(getQualifiedColumn());
            case Types.NVARCHAR: return rs.getString(getQualifiedColumn());
            case Types.NCHAR: return rs.getString(getQualifiedColumn());
            case Types.ROWID: return rs.getRowId(getQualifiedColumn());
            case Types.REF: return rs.getRef(getQualifiedColumn());
        }

        return rs.getObject(getQualifiedColumn());
    }

    public void setJoinMap(Mapper joinMap) {
        this.joinMap = joinMap;
    }

    public Mapper getJoinMap() {
        return joinMap;
    }
}
